/***************************************************************
*             Copyright (C) 2003 by SerComm Corp.
*                    All Rights Reserved.
*
*      Use of this software is restricted to the terms and
*      conditions of SerComm's software license agreement.
*
*                        www.sercomm.com
****************************************************************/

#ifndef  _IMG_CONFIG_H_
#define  _IMG_CONFIG_H_
#include <linux/buffer.h>

//#define	AIC_SampleRate 		8000	//8000, 16000 or 32000
#define	AIC_SampleRate 		16000	//8000, 16000 or 32000
//#define	AIC_SampleRate 		32000	//8000, 16000 or 32000
#define	DSP_BitR 		0	//0-G726_16K, 1-G726_24K, 2-G726_32k
					//3-G726_64K
					//!!! Same as maf.h AUDIO_CODEC !!!


#define  IMG_OK      0
#define  IMG_ERROR  -1

/* resolution */
#define  IMG_RES_120  1  /* 120 x 160 */
#define  IMG_RES_240  2  /* 240 x 320, default */
#define  IMG_RES_480  3  /* 480 x 640 */

/* compression rate */
#define  IMG_CMP_VHI   1  /* very high */
#define  IMG_CMP_HI    2  /* high */
#define  IMG_CMP_MID   3  /* middle, default */
#define  IMG_CMP_LOW   4  /* low */
#define  IMG_CMP_VLOW  5  /* very low */

/* time stamp and msg location */
#define  IMG_LOC_UL    1  /* upper left, default */
#define  IMG_LOC_UR    2  /* upper right */
#define  IMG_LOC_LL    3  /* lower left */
#define  IMG_LOC_LR    4  /* lower right */

/* image adjustment for hue, brightness, contrast */
#define  IMG_ADJ_MAX  64
#define  IMG_ADJ_DFT  32
#define  IMG_ADJ_MIN   0

/* rotation */
#define  IMG_ROT_0     0
#define  IMG_ROT_90    1
#define  IMG_ROT_180   2
#define  IMG_ROT_270   3

/* msg max length */
#define  IMG_MSG_LEN  20

/* audio volume */
#define  IMG_VOL_MIN   0
#define  IMG_VOL_DFT  16
#define  IMG_VOL_MAX  16

typedef struct IMG_Conf {
   char  res;  // IMG_RES_xxx, default IMG_RES_240
   char  comp; // compression rate (quality), default IMG_CMP_MID
   char  stmp; // time stamp, 0-disable(default), 1-enable
   char  text; // msg display, 0-disable(default), 1-enable
   char  msg[IMG_MSG_LEN+2]; //  IMG_MSG_LEN characters
   char  loc;  // time stamp and msg location, IMG_LOC_XX, default IMG_LOC_UL
   char  brit; // Brightness, IMG_ADJ_MIN~IMG_ADJ_MAX, default IMG_ADJ_DFT
   char  cnst; // Contrast, IMG_ADJ_MIN~IMG_ADJ_MAX, default IMG_ADJ_DFT
   char  atex; // Auto Exposure, 0-disable(default), 1-enable
   char  r;  // Red, IMG_ADJ_MIN~IMG_ADJ_MAX, default IMG_ADJ_DFT
   char  g;  // Green, IMG_ADJ_MIN~IMG_ADJ_MAX, default IMG_ADJ_DFT
   char  b;  // Blue, IMG_ADJ_MIN~IMG_ADJ_MAX, default IMG_ADJ_DFT
   char  rot;  // rotation, IMG_ROT_0, IMG_ROT_90,...
   char  en_a; // audio, 0-disable, 1-enable
   char  vol;  // audio volume, IMG_VOL_MIN~IMG_VOL_MAX, default IMG_VOL_DFT
   short rate; // frame rate, depends on resolution, default 15 fps
   short bitr; // bit rate, default 300. you must * 1024
   short sens; // motion detect sensitive, default 3
   short freq; // power line frequency, default 60Hz
   short emtm; // e-mail time interval in second, default 2 minutes
} IMG_Conf;

typedef struct IMG_INFO {
   VIDEO video;
   short   i;  // 1 - I frame, 0 - P frame
   short   a;  // 1 - audio, 0 - video
} IMG_INFO;

/* init IMG settings */
int IMG_init(void);
/* stop IMG */
int IMG_stop(void);
/* read IMG configuration data from the configuration file */
int IMG_GetConf(IMG_Conf *conf);
/* write IMG configuration data to the configuration file */
int IMG_SetConf(IMG_Conf *conf);
/* create an image account */
int IMG_open_acnt(int id);
/* close the image account */
int IMG_close_acnt(int id);
/* read image, the i_size does not include the size of IMG_INFO */
char * IMG_GetImg(long *i_size, int *i_index, int acnt);
/* release image */
int IMG_ReleaseImg(int i_index, int acnt);
/* motion detect enable/disable, 0-disable(default), 1-enable */
int IMG_MotDetc(int enable);
/* get the motion detect setting value */
int IMG_GetMotDetc(void);
/* run the video driver */
void act_v_drv(void);
/* motion detect handler is running */
void IMG_SetMotRun(void);
/* motion detect handler is stop */
void IMG_StopMotRun(void);

typedef struct IMG_AUD {
   unsigned short numChannels; // 1
   unsigned short samplesPerSec; // 8000
   unsigned short bytesPerSec; // 4000
} IMG_AUD;

typedef struct IMG_VID {
   unsigned short Res; // resolution before rotate, IMG_RES_xxx
   unsigned short FPS; // frame rate
   unsigned short BitRate; // bit rate. your bit rate must * 1024
   unsigned short Comp; // compression rate (quality)
} IMG_VID;

IMG_AUD * IMG_GetAudInfo(void);
IMG_VID * IMG_GetVidInfo(void);

typedef struct {
   short fi;  // force i-frame
   short rot; // rotate
   int   w;   // write index
   pid_t s;   // stop, pid - continue, 0 - stop
   int   m;   // number of buffers, max do not exceed 40
   long  b;   // buffer size
   short en_t; // enable time stamp
   short en_d; // enable msg display
   char  msg[IMG_MSG_LEN+2]; // msg to be displaied
   short en_m; // enable motion detect
   short m_r;  // motion handler is running
   short comp; // compression rate (quality)
   short en_a; // enable audio
   short sens; // motion detect sensitive
   short emtm; // e-mail time interval
//   short rsi;  // reset item, 0-dsp, 1-cmos, 2-both
   unsigned long rstv; // reset interval in mili second
//   short frm;  // 0-OSD ref, 1-OSD cap
} Img_S0;

typedef struct {
   int  cnt;  // counter for each buffer
   long sz;   // size of image. not include the size of IMG_INFO.
   char *loc; // location of buffer
} Img_S1;

#define IMG_S0   (VIDEO_DSP_PRIVATE_BASE+8)
#define IMG_S1   (IMG_S0 + sizeof(Img_S0))
#define IMG_MAX_FRMS   16

/*
img_s0
img_s1
buf_1
buf_2
...
buf_n
cur_frm
ref_frm
*/

#endif	/* _IMG_CONFIG_H_ */

/* configuration file sample
   [image]
   res=2
   comp=3
   stmp=0
   text=0
   msg=
   loc=1
   brit=32
   cnst=32
   hue=32
   rate=15
   rot=0
   motd=0
   sens=3
   freq=60
   emtm=2
*/
